@@ -33,47 +33,46 @@ Stats from running over a full database dump. We need to run verification over
 25586837 entity pairs, of which we 1346217/25586837 (or about 5%) are too
 ambiguous at this time.
 
-TODO: rerun to adjust formatting.
-
-    3450874 OK.TITLE_AUTHOR_MATCH
-    2619990 OK.SLUG_TITLE_AUTHOR_MATCH
-    2487633 Miss.YEAR
-    2434532 OK.WORK_ID
-    2085006 Miss.CONTRIB_INTERSECTION_EMPTY
-    1397420 Miss.SHARED_DOI_PREFIX
-    1355852 Miss.RELEASE_TYPE
-    1346217 OK.DUMMY
-    1145511 Miss.BOOK_CHAPTER
-    1009657 Miss.DATASET_DOI
-     996503 OK.PMID_DOI_PAIR
-     868951 OK.DATACITE_VERSION
-     796216 OK.DATACITE_RELATED_ID
-     704154 OK.FIGSHARE_VERSION
-     534963 OK.VERSIONED_DOI
-     343310 OK.TOKENIZED_AUTHORS
-     334974 OK.JACCARD_AUTHORS
-     293835 OK.PREPRINT_PUBLISHED
-     269366 Miss.COMPONENT
-     263626 Miss.SUBTITLE
-     224021 Miss.SHORT_TITLE
-     133811 Miss.CUSTOM_PREFIX_10_5860_CHOICE_REVIEW
-     122600 Miss.CUSTOM_PREFIX_10_7916
-      96935 Miss.PAGE_COUNT
-      79664 OK.CUSTOM_IEEE_ARXIV
-      46649 Miss.CUSTOM_PREFIX_10_14288
-      39797 Miss.JSTOR_ID
-      38598 OK.CUSTOM_BSI_UNDATED
-      18907 OK.CUSTOM_BSI_SUBDOC
-      15465 OK.DOI
-      13393 Miss.CUSTOM_IOP_MA_PATTERN
-      10378 Miss.CONTAINER
-       3081 Miss.BLACKLISTED
-       2504 Miss.BLACKLISTED_FRAGMENT
-       1273 Miss.APPENDIX
-       1063 Miss.TITLE_FILENAME
-	104 Miss.NUM_DIFF
-	  4 OK.ARXIV_VERSION
-
+Found   Status                  Reason
+--------------------------------------------------------------------------
+3450874 Status.EXACT    	Reason.TITLE_AUTHOR_MATCH
+2619990 Status.STRONG   	Reason.SLUG_TITLE_AUTHOR_MATCH
+2487633 Status.DIFFERENT        Reason.YEAR
+2434532 Status.EXACT    	Reason.WORK_ID
+2085006 Status.DIFFERENT        Reason.CONTRIB_INTERSECTION_EMPTY
+1397420 Status.DIFFERENT        Reason.SHARED_DOI_PREFIX
+1355852 Status.DIFFERENT        Reason.RELEASE_TYPE
+1290162 Status.AMBIGUOUS        Reason.UNKNOWN
+1145511 Status.DIFFERENT        Reason.BOOK_CHAPTER
+1009657 Status.DIFFERENT        Reason.DATASET_DOI
+ 996503 Status.STRONG   	Reason.PMID_DOI_PAIR
+ 868951 Status.EXACT    	Reason.DATACITE_VERSION
+ 796216 Status.STRONG   	Reason.DATACITE_RELATED_ID
+ 704154 Status.STRONG   	Reason.FIGSHARE_VERSION
+ 534963 Status.STRONG   	Reason.VERSIONED_DOI
+ 343310 Status.STRONG   	Reason.TOKENIZED_AUTHORS
+ 334974 Status.STRONG   	Reason.JACCARD_AUTHORS
+ 293835 Status.STRONG   	Reason.PREPRINT_PUBLISHED
+ 269366 Status.DIFFERENT        Reason.COMPONENT
+ 263626 Status.DIFFERENT        Reason.SUBTITLE
+ 224021 Status.AMBIGUOUS        Reason.SHORT_TITLE
+ 152990 Status.DIFFERENT        Reason.PAGE_COUNT
+ 133811 Status.AMBIGUOUS        Reason.CUSTOM_PREFIX_10_5860_CHOICE_REVIEW
+ 122600 Status.AMBIGUOUS        Reason.CUSTOM_PREFIX_10_7916
+  79664 Status.STRONG   	Reason.CUSTOM_IEEE_ARXIV
+  46649 Status.DIFFERENT        Reason.CUSTOM_PREFIX_10_14288
+  39797 Status.DIFFERENT        Reason.JSTOR_ID
+  38598 Status.STRONG   	Reason.CUSTOM_BSI_UNDATED
+  18907 Status.STRONG   	Reason.CUSTOM_BSI_SUBDOC
+  15465 Status.EXACT    	Reason.DOI
+  13393 Status.DIFFERENT        Reason.CUSTOM_IOP_MA_PATTERN
+  10378 Status.DIFFERENT        Reason.CONTAINER
+   3081 Status.AMBIGUOUS        Reason.BLACKLISTED
+   2504 Status.AMBIGUOUS        Reason.BLACKLISTED_FRAGMENT
+   1273 Status.AMBIGUOUS        Reason.APPENDIX
+   1063 Status.DIFFERENT        Reason.TITLE_FILENAME
+    104 Status.DIFFERENT        Reason.NUM_DIFF
+      4 Status.STRONG   	Reason.ARXIV_VERSION
 """
 
 import collections
@@ -82,34 +81,32 @@ import json
 import operator
 import re
 import sys
-from typing import Counter, Dict, Tuple
+from typing import Counter, Dict, Tuple, Type
 
+from fatcat_openapi_client import ReleaseEntity
 from glom import PathAccessError, glom
 
 from fuzzycat.common import Reason, Status
 from fuzzycat.data import (CONTAINER_NAME_BLACKLIST, PUBLISHER_BLACKLIST, TITLE_BLACKLIST,
                            TITLE_FRAGMENT_BLACKLIST)
+from fuzzycat.entities import entity_to_dict
 from fuzzycat.utils import (author_similarity_score, contains_chemical_formula, dict_key_exists,
-                            has_doi_prefix, jaccard, num_project, parse_page_string, slugify_string)
+                            doi_prefix, has_doi_prefix, jaccard, num_project, parse_page_string,
+                            slugify_string)
 
 Verify = collections.namedtuple("Verify", "status reason")
 
 
 class GroupVerifier:
     """
-    Verifier.
-
-    Within a group, we could have multiple sub clusters, e.g.
-
-    > [AABAB]
-
-    We would need to compare each possible pair and decide whether they are the
-    same.
+    Given an iterable (lines) from clustering step, run verification on every
+    pair of the cluster, e.g. a cluster of four elements will yield nCr(4, 2) =
+    10 pairs.
     """
     def __init__(self,
                  iterable: collections.abc.Iterable,
                  max_cluster_size: int = 10,
-                 verbose=True):
+                 verbose=False):
         self.iterable: collections.abc.Iterable = iterable
         self.max_cluster_size: int = max_cluster_size
         self.verbose: bool = verbose
@@ -150,6 +147,10 @@ class GroupVerifier:
         self.counter["total"] = sum(v for _, v in self.counter.items())
 
 
+def verify_release_entities(a: ReleaseEntity, b: ReleaseEntity, min_title_length=5) -> Type[Verify]:
+    return verify(entity_to_dict(a), entity_to_dict(b), min_title_length=min_title_length)
+
+
 def verify(a: Dict, b: Dict, min_title_length=5) -> Tuple[str, str]:
     """
     Compare two entities (dicts), return tuple of match status and reason.
@@ -160,11 +161,15 @@ def verify(a: Dict, b: Dict, min_title_length=5) -> Tuple[str, str]:
 
     The cases are relatively independent, so order should not matter, but that
     can be pinned down much more clearly.
+
+    TODO: add wrapper release entities
     """
 
     # A few items have the same DOI.
     try:
-        if glom(a, "ext_ids.doi") == glom(b, "ext_ids.doi"):
+        a_doi = glom(a, "ext_ids.doi")
+        b_doi = glom(b, "ext_ids.doi")
+        if a_doi is not None and a_doi == b_doi:
             return Verify(Status.EXACT, Reason.DOI)
     except PathAccessError:
         pass
@@ -178,9 +183,6 @@ def verify(a: Dict, b: Dict, min_title_length=5) -> Tuple[str, str]:
     b_title = b.get("title", "") or ""
     b_title_lower = b_title.lower()
 
-    assert isinstance(a_title, str)
-    assert isinstance(b_title, str)
-
     if len(a_title) < min_title_length:
         return Verify(Status.AMBIGUOUS, Reason.SHORT_TITLE)
     if a_title_lower in TITLE_BLACKLIST:
@@ -464,7 +466,7 @@ def verify(a: Dict, b: Dict, min_title_length=5) -> Tuple[str, str]:
             # preprint and published work may not be published in the same
             # year; compromise allow a small gap
             if a_release_year and b_release_year and abs(int(a_release_year) -
-                                                         int(b_release_year)) > 1:
+                                                         int(b_release_year)) > 4:
                 return Verify(Status.DIFFERENT, Reason.YEAR)
             return Verify(Status.EXACT, Reason.TITLE_AUTHOR_MATCH)
 
@@ -525,7 +527,7 @@ def verify(a: Dict, b: Dict, min_title_length=5) -> Tuple[str, str]:
         b_doi = glom(b, "ext_ids.doi")
 
         if a_container_id == b_container_id and a_doi != b_doi and not has_doi_prefix(
-                a_doi, "10.1126"):
+                a_doi, "10.1126") and doi_prefix(a_doi) == doi_prefix(b_doi):
             return Verify(Status.DIFFERENT, Reason.SHARED_DOI_PREFIX)
     except PathAccessError:
         pass
@@ -540,6 +542,7 @@ def verify(a: Dict, b: Dict, min_title_length=5) -> Tuple[str, str]:
         # explodes.
         a_trimmed = sorted(a_slug_authors)[:5]
         b_trimmed = sorted(b_slug_authors)[:5]
+        num_authors = min(len(a_trimmed), len(b_trimmed))
         for a, b in itertools.product(a_trimmed, b_trimmed):
             scores.append(Score(a, b, author_similarity_score(a, b)))
         # TODO: less arbitrary metric and threshold
@@ -550,7 +553,7 @@ def verify(a: Dict, b: Dict, min_title_length=5) -> Tuple[str, str]:
                 top_scores.append(sorted_scores[0].value)
         if len(top_scores) > 0:
             avg_score = sum(top_scores) / len(top_scores)
-            if avg_score > 0.5:
+            if (num_authors < 3 and avg_score > 0.9) or (num_authors >= 3 and avg_score > 0.5):
                 return Verify(Status.STRONG, Reason.TOKENIZED_AUTHORS)
             else:
                 pass
@@ -593,4 +596,15 @@ def verify(a: Dict, b: Dict, min_title_length=5) -> Tuple[str, str]:
     except (ValueError, PathAccessError):
         pass
 
-    return Verify(Status.AMBIGUOUS, Reason.DUMMY)
+    # A variant of translated titles, e.g.
+    # https://fatcat.wiki/release/search?q=%22A+nova+classifica%C3%A7%C3%A3o+dos+tumores+da+mama+%22
+    try:
+        a_container_id = glom(a, "container_id")
+        b_container_id = glom(b, "container_id")
+        if a_authors == b_authors and a_container_id == b_container_id and a_release_year == b_release_year and a_title != b_title and (
+                a_title in b_title or b_title in a_title):
+            return Verify(Status.STRONG, Reason.TITLE_ARTIFACT)
+    except PathAccessError:
+        pass
+
+    return Verify(Status.AMBIGUOUS, Reason.UNKNOWN)
